<?php
namespace Financeiro\Logic\BaixaBoleto;
use \Financeiro\Entities\Baixarecebida;
use Geral\Util\Ambiente;
use \Geral\Util\DateUtil;
use \Financeiro\Entities\Contareceber;
use Geral\Util\NovoCodigo;
use Geral\Util\TratamentoDeMensagens;
use \Setforms2\Shared\Registry;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use \Financeiro\Entities\Movbancario;
use Financeiro\Entities\Ajctreceber;
use Financeiro\Entities\Boletreceber;
use Financeiro\Logic\AtualizaFluxodeCaixa\FluxodeCaixa;
use \Setforms2\Shared\ORM\DateTime;
use Financeiro\Entities\Movfluxo;
class BaixaBoleto{

	public static function updateBaixa($valor, $nrmovbanc){
		$baixa = tk_get_env_var('nova_baixa');

		try{
            $baixarecebida = Baixarecebida::findOneByNrorgAndCdfilialAndNrbaixarec($baixa['NRORG'][1], $baixa['CDFILIAL'][1], $baixa['NRBAIXAREC'][1]);
            if(!$baixarecebida)
			    $baixarecebida = new Baixarecebida();
			$baixarecebida->setNrbaixarec($baixa['NRBAIXAREC'][1]);
            $baixarecebida->setNrmovbanc($nrmovbanc);
			$baixarecebida->setCdfilial($baixa['CDFILIAL'][1]);
			$baixarecebida->setCdempresa($baixa['CDEMPRESA'][1]);
			$dtlancbaixar = self::formataData($baixa['DTLANCBAIXAR'][1]);
  			$baixarecebida->setDtlancbaixar($dtlancbaixar);
  			$dtbaixarec = self::formataData($baixa['DTBAIXAREC'][1]);
			$baixarecebida->setDtbaixarec($dtbaixarec);
			$dtinclusao = DateUtil::getDataAtual();
			$baixarecebida->setDtinclusao($dtinclusao);
			$baixarecebida->setCdoperador($baixa['CDOPERADOR'][1]);
			$baixarecebida->setIdstatusbrec($baixa['IDSTATUSBREC'][1]);
			$baixarecebida->setCdreducont($baixa['CDREDUCONT'][1]);
			$baixarecebida->setCdtpbaixarec($baixa['CDTPBAIXAREC'][1]);
			$baixarecebida->setNrorg($baixa['NRORG'][1]);
			$baixarecebida->setIdstatusbrec($baixa['IDSTATUSBREC'][1]);
			$baixarecebida->setIdstatusctbrec($baixa['IDSTATUSCTBREC'][1]);
			$baixarecebida->setDsbaixarec($baixa['DSBAIXAREC'][1]);
			$baixarecebida->setCdoperinclusao(Ambiente::getCdOperador());
			$baixarecebida->setNrorginclusao(Ambiente::getNrorg());
			$baixarecebida->setVrbaixarec($valor);

			$baixarecebida->persist();
	        Registry::getEntityManager()->flush();

	    } catch (\Exception $e) {
            throw $e;
        }
	}

    public static function geraMovimentacao($cdreducont, $dtmovbanc, $vrmovbanc){
        try {
            $nrmovbanc = NovoCodigo::geraCodigo(Ambiente::getNrorg(), 'MOVBANCARIO' . $cdreducont);

            $movbancario = new Movbancario;
            $movbancario->setCdreducont($cdreducont);
            $movbancario->setNrmovbanc($nrmovbanc);
            $dtmovbanc = self::formataData($dtmovbanc);
            $movbancario->setDtmovbanc($dtmovbanc);
            $movbancario->setIdmovbanc('C');
            $movbancario->setDshismbanc('bxRec');
            $movbancario->setVrmovbanc($vrmovbanc);
            $movbancario->setIdorigmbanc('R');
            $movbancario->setIdstatusctbbanc('P');
            $movbancario->setIdativo('S');
            $movbancario->setNrorg(Ambiente::getNrorg());
            $movbancario->setDtinclusao(DateUtil::getDataAtual());
            $movbancario->persist();

            Registry::getEntityManager()->flush();

            return $nrmovbanc;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function contabilizaBaixa($cdempresa, $cdfilial, $filiproc, $nrbaixa){
        try{
            $nrlancctb = '';
            $procedure = new Procedure('CTB_REC_BAIXA');
            $procedure->addParam(new Param('P_CDEMPRESA', Param::PARAM_INPUT, $cdempresa, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_FILIPROC', Param::PARAM_INPUT, $filiproc, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRBAIXA', Param::PARAM_INPUT, $nrbaixa, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, $nrlancctb, Param::PARAM_TYPE_STR,12));
            $nrlancctb = $procedure->call();
            return $nrlancctb['P_NRLANCCTB'];
        }catch (\Exception $e){
            throw $e;
        }
    }

    public static function updateContasReceber($cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $idstatrec, $idmovfluxo, $dtbaixa){
        try {

            /** @var $conta Contareceber */
            $conta = Contareceber::findOneByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarecAndNrorg(
                $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, Ambiente::getNrorg()
            );

            $dtbaixa = self::formataData($dtbaixa);
            $conta->setDtmovfluxo($dtbaixa);
            $conta->setIdstatrec($idstatrec);
            $conta->setIdmovfluxo($idmovfluxo);
            $conta->persist();

            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }
	
	public static function updateContaReceberBaixado($nrlanbolrec, $cdfilial){
		try {
            /** @var $conta Contareceber */
			$conta = Contareceber::findOneByCdfilibolrecAndNrlanbolrecAndNrorg($cdfilial, $nrlanbolrec, Ambiente::getNrorg());
			$conta->setIdstatrec('A');
			$conta->persist();
			Registry::getEntityManager()->flush();
				
		} catch (\Exception $e) {
			throw $e;
		}
	}

    public static function atualizaBoletosBaixaPlanejada($boletos, $cdfilial, $nrbaixarec, $tipobaixa, $dataBaixa){
        try {
            foreach ($boletos as $boleto) {
                /** @var $boletoEntity Boletreceber */
                $boletoEntity = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                    $boleto["NRLANBOLREC"], $boleto["CDFILIAL"], $boleto["NRORG"]
                );
                $boletoEntity->setCdfilibairec($cdfilial);
                $boletoEntity->setNrbaixarec($nrbaixarec);
                $boletoEntity->setIdstatbrec("P");
                $boletoEntity->setCdtpbaixarec($tipobaixa);
                $boletoEntity->setDtliqbolrec(self::formataData($dataBaixa));
                $boletoEntity->persist();
            }

            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function atualizaContaBaixaPlanejada($boletos, $dataBaixa){
        foreach($boletos as $boleto){
            $contas = Contareceber::findByCdfilibolrecAndNrlanbolrecAndNrorg(
                $boleto["CDFILIAL"], $boleto["NRLANBOLREC"], $boleto["NRORG"]
            );

            /** @var $conta Contareceber*/
            foreach($contas as $conta){
                FluxodeCaixa::atualizaFluxo(
                    $conta->getCdempresa(), $conta->getCdfilial(), $conta->getCdclassfina(), self::formataData($dataBaixa),
                    $boleto["VRFINALBREC"], "E"
                );

                $conta->setIdmovfluxo("E");
                $conta->setIdstatrec("P");
                $conta->setDtmovfluxo(self::formataData($dataBaixa));
                $conta->persist();
            }
        }

        Registry::getEntityManager()->flush();
    }

    public static function updateContaReceberBoleto($nrlanbolrec, $cdfilial){
        try {
            /** @var $conta Contareceber */
            $contas = Contareceber::findByCdfilibolrecAndNrlanbolrecAndNrorg($cdfilial, $nrlanbolrec, Ambiente::getNrorg());
            foreach($contas as $conta){
                self::atualizaFluxodeCaixa($conta);
                $conta->setIdstatrec('A');
                $conta->setIdmovfluxo('E');
                $conta->persist();
                Registry::getEntityManager()->flush();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function atualizaFluxodeCaixa($conta){
        /** @var $conta Contareceber */
        FluxodeCaixa::atualizaFluxo(
            $conta->getCdempresa(),
            $conta->getCdfimovflrec(),
            $conta->getCdclassfina(),
            $conta->getDtmovfluxo(),
            -($conta->getVrorigrec()),
            $conta->getIdmovfluxo()
        );
        FluxodeCaixa::atualizaFluxo(
            $conta->getCdempresa(),
            $conta->getCdfimovflrec(),
            $conta->getCdclassfina(),
            $conta->getDtmovfluxo(),
            $conta->getVrorigrec(),
            'E'
        );
    }

    public static function updateBaixaBoleto($nrorg, $cdfilial, $nrbaixarec, $valorBoleto = NULL, $nrmovbanc = NULL){
        try {
            /** @var $baixa Baixarecebida */
            $baixa = Baixarecebida::findOneByNrorgAndCdfilialAndNrbaixarec(
                $nrorg, $cdfilial, $nrbaixarec
            );
            $baixa->setNrmovbanc($nrmovbanc);
            if($valorBoleto)
                $baixa->setVrbaixarec($baixa->getVrbaixarec() + $valorBoleto);
            $baixa->persist();

            return $baixa;
        } catch (\Exception $e) {
            throw $e;
        }
    }

	public static function updateBoleto($boleto){
        try {

            $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $boleto['NRORG']);
            /** @var $boleto Boletreceber */
            if ($boleto->getCdbarrarec() != null)
                $boleto->setIdstatbrec('E');
            else
                $boleto->setIdstatbrec('A');
            $boleto->setNrbaixarec(NULL);
            $boleto->setCdfilibairec(NULL);
            $boleto->setDtliqbolrec(NULL);
            if ($boleto->getNrcnabrecrem())
                $boleto->setIdsitucnabr('10');
            else
                $boleto->setIdsitucnabr('00');

            $boleto->persist();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function deletaMovimentacao($nrorg, $cdreducont, $nrmovbanc){
        try {
            /** @var $movbanc Movbancario */
            $movbanc = Movbancario::findOneByNrorgAndCdreducontAndNrmovbanc($nrorg, $cdreducont, $nrmovbanc);
            if($movbanc)
                $movbanc->delete();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateContaReceber($cdfilial, $nrlanbolrec, $dtbaixarec){
        try {
            /** @var $conta Contareceber */
            $contas = Contareceber::findByCdfilibolrecAndNrlanbolrecAndNrorg(
                $cdfilial, $nrlanbolrec, Ambiente::getNrorg()
            );
            foreach($contas as $conta){
                $idfluxo = $conta->getIdmovfluxo() ? $conta->getIdmovfluxo() :
                    Movfluxo::TIPO_MOVIMENTO_EMPENHO;

                $dtbaixarec = self::formataData($dtbaixarec);

                FluxodeCaixa::atualizaFluxo(
                    $conta->getCdempresa(), $conta->getCdfimovflrec(),
                    $conta->getCdclassfina(), $conta->getDtmovfluxo(),
                    ($conta->getVratuarec() * - 1), 'E', 'Entrada'
                );
                FluxodeCaixa::atualizaFluxo(
                    $conta->getCdempresa(), $conta->getCdfimovflrec(),
                    $conta->getCdclassfina(), $dtbaixarec,
                    $conta->getVratuarec(), 'R', 'Entrada'
                );
                self::updateAjusteConta(
                    $conta->getCdfilial(), $conta->getNrlanctonf(), $conta->getNrseqvenrec(),
                    $conta->getNrparcelarec(), 'I', 'E', $idfluxo, $dtbaixarec
                );

                $conta->setDtmovfluxo($dtbaixarec);
                $conta->setIdmovfluxo('R');
                $conta->setIdstatrec('B');
            }
            Registry::getEntityManager()->flush();
            return array(
                'NRLANCTONF'  => $conta->getNrlanctonf(),
                'CDFILIAL'    => $conta->getCdfilial(),
                'CDEMPRESA'   => $conta->getCdempresa(),
                'CDCLASSFINA' => $conta->getCdclassfina(),
                'DTMOVFLUXO'  => $conta->getDtmovfluxo(),
                'VRATUAREC'   => $conta->getVratuarec()
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateAjusteConta($cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $idModo, $idBaixa, $idfluxo, $dtbaixa){
        try {

            $idtipo = $idBaixa == 'P' ? Movfluxo::TIPO_MOVIMENTO_EMPENHO : Movfluxo::TIPO_MOVIMENTO_REAL;

            /** @var $ajustes Ajctreceber */
            $ajustes = Ajctreceber::findByNrorgAndCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarec(
                Ambiente::getNrorg(), $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec
            );
            /** @var $ajuste Ajctreceber */
            if($ajustes){
                foreach($ajustes as $ajuste){

                    if($idModo == 'E')
                        $dtbaixa = $ajuste->getDtajusterec();

                    $ajfluxo = $ajuste->getIdmovfluxo() ? $ajuste->getIdmovfluxo() : 'E';

                    if($ajuste->getAjuctrec()->getIdajuctrec() == \Geral\Entities\Tipoajpagar::ID_AJUSTE_ACRESCIMO){


                        // Desfaz o movimento de fluxo estimado
                        FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntr(),
                            $ajuste->getCdfimfajcntr(),
                            $ajuste->getCdclassfina(),
                            $ajuste->getDtmovfluxo(),
                            $ajuste->getVrajusterec() * -1,
                            $ajfluxo,
                            'Entrada'
                        );

                        // Inclui o movimento de fluxo real
                        FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntr(),
                            $ajuste->getCdfimfajcntr(),
                            $ajuste->getCdclassfina(),
                            $dtbaixa,
                            $ajuste->getVrajusterec(),
                            $idtipo,
                            'Entrada'
                        );
                    }
                    else{

                        // Desfaz o movimento de fluxo estimado
                        FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntr(),
                            $ajuste->getCdfimfajcntr(),
                            $ajuste->getCdclassfina(),
                            $ajuste->getDtmovfluxo(),
                            $ajuste->getVrajusterec(),
                            $ajfluxo,
                            'Entrada'
                        );

                        // Inclui o movimento de fluxo real
                        FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntr(),
                            $ajuste->getCdfimfajcntr(),
                            $ajuste->getCdclassfina(),
                            $dtbaixa,
                            $ajuste->getVrajusterec() * -1,
                            $idfluxo,
                            'Entrada'
                        );
                    }
                    self::atualizaAjuste($ajuste, $idtipo, $dtbaixa);

                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function atualizaAjuste(Ajctreceber $ajuste, $idmovfluxo, DateTime $dtmovfluxo){

        $ajuste->setIdmovfluxo($idmovfluxo);
        $ajuste->setDtmovfluxo($dtmovfluxo);
        $ajuste->persist();
        Registry::getEntityManager()->flush();

    }

    public static function updateBaixaCnab($nrmovbanc, $cdfilial, $nrbaixarec){
        try {
            /** @var $baixa Baixarecebida */
            $baixa = Baixarecebida::findOneByNrorgAndCdfilialAndNrbaixarec(Ambiente::getNrorg(), $cdfilial, $nrbaixarec);
            $baixa->setNrmovbanc($nrmovbanc);
            $baixa->setIdstatusbrec('E');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function validaUnicoBoleto($nrorg, $cdfilibairec, $nrbaixarec){
        try {
            $boletos = Boletreceber::findByNrorgAndCdfilibairecAndNrbaixarec($nrorg, $cdfilibairec, $nrbaixarec);
            if (count($boletos) == 1)
                throw new \Exception('Baixa possui apenas um boleto. Exclua a baixa!');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function validaConciliacao($nrorg, $cdreducont, $nrmovbanc){
        try {
            /** @var $movbanc Movbancario */
            $movbanc = Movbancario::findOneByNrorgAndCdreducontAndNrmovbanc($nrorg, $cdreducont, $nrmovbanc);
            if ($movbanc) {
                if ($movbanc->getIdstconext() != 0) {
                    throw new \Exception('Exclusão não permitida! Movimento bancário da baixa foi conciliado.');
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

	public static function trataData($data, $datahr){
		$datahora = $datahr;
		if($data instanceof \DateTime == false){
	    	if(strlen($datahora) == 10)
		    	$data = $data.'00:00:00';
		    $datahora = DateUtil::getDataDeString($datahora);
		}
		$datahora = DateUtil::truncateData($datahora);
		return $datahora;
	}

    public static function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}