<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 14/11/2014
 * Time: 16:42
 */

namespace Folha\Logic\ValesBeneficios\IntegracaoEletronicaVT;


abstract class Leitor
{
    protected $formatosAceitos = "";
    /**
     * Abre o arquivo que contém a informação de saldo disponível do vale-transporte.
     * @param String $caminho
     * @return mixed
     */
    abstract protected function abrirArquivo($caminho);

    /**Valida o formato do arquivo que foi enviado pelo usuário.
     * @param String $caminho
     * @return bool
     * @throws ExcessaoIntegracaoVT
     * @throws \Exception
     */
    private function validaFormato( $caminho){
        try {
            $caminho = strtoupper($caminho);
            if (!preg_match("/\.(".$this->formatosAceitos.")/", $caminho )) {
                throw ExcessaoIntegracaoVT::extensaoInvalida();
            }
        } catch (ExcessaoIntegracaoVT $e) {
            if($e->getCode() == ExcessaoIntegracaoVT::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * valida o formato e a consistência da informação.
     * @param \PHPExcel $info
     * @return boolean
     * @throws ExcessaoIntegracaoVT
     * @throws \Exception
     */
    private function validaInfo( $info)
    {
        try{
            foreach($info as $reg){
                if(empty($reg['NUMERO_CARTAO']) || empty($reg['SALDO_CARTAO'])){
                    throw ExcessaoIntegracaoVT::infoInvalida();
                }
            }

        }catch (ExcessaoIntegracaoVT $e){
            if($e->getCode() == ExcessaoIntegracaoVT::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Cria um Array contendo Arrays no formato:
     * [
     *     NUMERO_CARTAO : '',
     *     SALDO_CARTAO : ''
     * ]
     * @param mixed $info
     * @return Array
     * @throws ExcessaoIntegracaoVT
     * @throws \Exception
     */
    abstract protected function preparaInfo( $info);

    /**
     * Executa as funções para adquirir, tratar e disponibiliza a informação.
     * @param String $caminho
     * @return array
     * @throws ExcessaoIntegracaoVT
     * @throws \Exception
     */
    public function getInfo( $caminho)
    {
        $this->validaFormato( $caminho);
        $info = $this->abrirArquivo($caminho);
        $info = $this->preparaInfo($info);
        $this->validaInfo( $info);

        return $info;

    }

} 